/*
 * rotor.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <string.h>
#include <fcntl.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "diseqc.h"

int FromString(const char *s)
{
  enum eSourceType {
    stNone  = 0x0000,
    stCable = 0x4000,
    stSat   = 0x8000,
    stTerr  = 0xC000,
    st_Mask = 0xC000,
    st_Neg  = 0x0800,
    st_Pos  = 0x07FF,
    };
  int type = stNone;
  switch (toupper(*s)) {
    case 'C': type = stCable; break;
    case 'S': type = stSat;   break;
    case 'T': type = stTerr;  break;
    case '0' ... '9': type = *s - '0'; break; // backward compatibility
    default: printf("ERROR: unknown source key '%c'", *s);
             return stNone;
    }
  int code = type;
  if (type == stSat) {
     int pos = 0;
     bool dot = false;
     bool neg = false;
     while (*++s) {
           switch (toupper(*s)) {
             case '0' ... '9': pos *= 10;
                               pos += *s - '0';
                               break;
             case '.':         dot = true;
                               break;
             case 'E':         neg = true; // fall through to 'W'
             case 'W':         if (!dot)
                                  pos *= 10;
                               break;
             default: printf("ERROR: unknown source character '%c'", *s);
                      return stNone;
             }
           }
     if (neg)
        pos |= st_Neg;
     code |= pos;
     }
  return code;
}

const char *HelpPages(void)
{
  static const char HelpPages[] = 
    "DRIVE east | west\n"
    "    Drives the rotor east or west.\n"
    "HALT\n"
    "    Halts the rotor.\n"
    "STEPS east | west [ <steps> ]\n"
    "    Drives the rotor one (or the given) steps east or west.\n"
    "GOTO <position>\n"
    "    Lets the motor drive to the position.\n"
    "STORE <position>\n"
    "    Stores the current position.\n"
    "RECALC [ <satellite position> ]\n"
    "    Recalculates the satellite positions.\n"
    "LIMITS on | off\n"
    "    Enable / Disable Softlimits.\n" 
    "SETL east | west\n"
    "    Sets east / west softlimit.\n"
    "GOTOX <source>\n"
    "    Drives the rotor to the given source (like S13E)\n";
  return HelpPages;
}

void Command(const char *Command, const char *Option)
{
  if (strcasecmp(Command, "DRIVE") == 0) {
    if (*Option) {
      if (strcasecmp(Option, "east") == 0) {
         DiseqcCommand(DriveEast);
         printf("Started to drive the rotor eastward\n");
         }
      else if (strcasecmp(Option, "west") == 0) {
         DiseqcCommand(DriveWest);
         printf("Started to drive the rotor westward\n");
         }
      else {
         printf("Unknown option: \"%s\"\n", Option);
         }
      }
    else {
      printf("No argument given for command DRIVE\n");
      }
    }  
  else if (strcasecmp(Command, "HALT") == 0) {
    DiseqcCommand(Halt);
    printf("Rotor was halted\n");
    }
  else if (strcasecmp(Command, "STEPS") == 0) {
    if (*Option) {
      char buf[strlen(Option) + 1];
      strcpy(buf, Option);
      const char *delim = " \t";
      char *strtok_next;
      char *p = strtok_r(buf, delim, &strtok_next);
      if (strcasecmp(p, "east") == 0 || strcasecmp(p, "west") == 0) {
          char *b = strtok_r(NULL, delim, &strtok_next);
          int steps=1;
          if (b)
            steps=atoi(b);
          if (strcasecmp(p, "east") == 0) {
             DiseqcCommand(DriveStepsEast,steps);
             printf("Drove %d steps eastward.\n",steps);
             }
          else {
             DiseqcCommand(DriveStepsWest,steps);
             printf("Drove %d steps westward.\n",steps);
             }
         }
      else {
         printf("Unknown option: \"%s\"\n", Option);
         }
      }
    else {
      printf("No argument given for command STEPS\n");
      }
    }
  else if (strcasecmp(Command, "GOTO") == 0) {
    if (*Option) {
      DiseqcCommand(Goto,atoi(Option));
      printf("Goto position %d\n",atoi(Option));
      }
    else
      printf("No argument given for command GOTO\n");
    }
  else if (strcasecmp(Command, "STORE") == 0) {
    if (*Option) {
      DiseqcCommand(Store,atoi(Option));
      printf("Stored position %d\n",atoi(Option));
      }
    else
      printf("No argument given for command STORE\n");
    }
  else if (strcasecmp(Command, "RECALC") == 0) {
    int number = 0;
    if (*Option) 
      number = atoi(Option);
    DiseqcCommand(Recalc,number);
    printf("Recalculated positions\n");
    }
  else if (strcasecmp(Command, "LIMITS") == 0) {
    if (*Option) {
      if (strcasecmp(Option, "on") == 0) {
         DiseqcCommand(LimitsOn);
         printf("Turned limits on\n");
         }
      else if (strcasecmp(Option, "off") == 0) {
         DiseqcCommand(LimitsOff);
         printf("Turned limits off\n");
         }
      else {
         printf("Unknown option: \"%s\"\n", Option);
         }
      }
    else {
      printf("No argument given for command LIMITS\n");
      }
    }
  else if (strcasecmp(Command, "SETL") == 0) {
    if (*Option) {
      if (strcasecmp(Option, "east") == 0) {
         DiseqcCommand(SetEastLimit);
         printf("Set east limit\n");
         }
      else if (strcasecmp(Option, "west") == 0) {
         DiseqcCommand(SetWestLimit);
         printf("Set west limit\n");
         }
      else {
         printf("Unknown option: \"%s\"\n", Option);
         }
      }
    else {
      printf("No argument given for command SETL\n");
      }
    }
  else if (strcasecmp(Command, "GOTOX") == 0) {
    if (*Option) {
      int i=FromString(Option);
      GotoX(i); 
      printf("Goto angular position %s\n",Option);
      }
    else
      printf("No argument given for command GOTOX\n");
    }
  else printf("Command unknown: %s\n",Command);
}

int Long=0,Lat=0,Adapter=0;

int main(int argc, char *argv[])
{
  FILE *f = fopen("rotor.conf", "r");
  char b[256];
  int v,n,h;
  while (fscanf(f,"%s %d.%d%c",b,&v,&n,&h)!=EOF)
  {
    if (strcmp(b,"Longitude") == 0)
      Long=(v*10+n) * ((toupper(h)=='E') ? -1 : 1);
    if (strcmp(b,"Latitude") == 0)
      Lat=(v*10+n) * ((toupper(h)=='S') ? -1 : 1);
    if (strcmp(b,"Adapter") == 0)
      Adapter = v;        
  }
  fclose(f);
  if (argc == 1)
  {
    printf("No arguments given\n");
    return -1;
  }
  if (strcmp(argv[1],"-h") == 0 || strcmp(argv[1],"--help") == 0)
  {
    printf("%s\n",HelpPages());
    return 0;
  }
  Command(argv[1],argv[2]);
  return 0;
}
